# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['aush']
setup_kwargs = {
    'name': 'aush',
    'version': '0.9.4',
    'description': 'Pythonic subprocess library',
    'long_description': '# aush\n\nPythonic subprocess library\n\n# Introduction\n\n`aush` is a Python library to make calling other programs and getting their output as easy as possible. It\'s intended to be the best possible melding of shell scripting and Python. It\'s *aushome*.\n\n# Philosophy\n\nCommand line programs are basically “functions” available to call from any other language, whose signature looks like: `list[str] -> (code: smallint, stdout: stream, stedrr: stream)`. That\'s what calls like [execve](https://linux.die.net/man/2/execve) are doing underneath.\n\nThe goal of this library is to make calling those functions as natural as possible within Python.\n\n# Motivation\n\nSo often I\'ve found myself writing a small Bash/shell script that just calls some programs and maybe does some conditionals. Naturally, it has to start with\n\n```\n#!/usr/bin/env bash\nset -Eeuxo pipefail\n```\n\nfor a "strict mode" that has some of the same failure properties you expect a program to have, such as that your program doesn\'t just keep running when a command fails, it won\'t silently proceed if you use undeclared variables, and so on.\n\nThen it frequently occurs to want to do something "hard" in shell, like string processing or data structures or even command-line argument parsing.\nPython has `argparse` in the stdlib, I don\'t want to mess with `getopt`.\nI want to be able to bust out some Pandas DataFrames or Requests requests, read json or whatever.\n\nSo often when I start with Bash, I wind up transitioning the script to Python, and that often feels *terrible*.\nPipes become an unnatural operation.\nI can\'t just "redirect to a file" I need to know how to call `subprocess.run` to capture stdout (there are a few different ways), open and close the file, and so on.\nInstead, `aush` makes the transition from shell to Python feel *better*.\n\nHere\'s the [origin thread on lobste.rs](https://lobste.rs/s/p1hict/zxpy_tool_for_shell_scripting_python#c_agohpx).\n\n# Installation\n\n```pip install aush```\n\n# Examples\n\nI recently make a [create-python](https://github.com/kbd/setup/blob/master/HOME/bin/create-python) program that will initialize a Python project the same way every time, as a [Poetry](https://python-poetry.org/) project with the same set of default dependencies, linters, and so on.\n\nAt first I created a shell script. Here\'s a portion:\n\n```bash\npoetry init \\\n  --no-interaction \\\n  --name="$1" \\\n  --author="$(git config --get user.name)" \\\n  --dev-dependency=pytest \\\n  --dev-dependency=ipython \\\n  --dev-dependency=pylint \\\n  --dev-dependency=mypy \\\n  --dev-dependency=black \\\n  --dev-dependency=pudb\n\npoetry install\n```\n\nHere\'s how that converts to Python:\n\n```python\ncmd = [\'git\', \'config\', \'--get\', \'--global\', \'user.name\']\nusername = run(cmd, check=True, capture_output=True, text=True).stdout.strip()\n\ncmd = [\n  \'poetry\', \'init\',\n  \'--no-interaction\',\n  \'--name\', sys.argv[1],\n  \'--author\', username,\n  \'--dev-dependency=pytest\',\n  \'--dev-dependency=ipython\',\n  \'--dev-dependency=pylint\',\n  \'--dev-dependency=mypy\',\n  \'--dev-dependency=black\',\n  \'--dev-dependency=pudb\',\n]\nrun(cmd, check=True)\nrun([\'poetry\', \'install\'], check=True)\n```\n\nHere\'s how that looks now using `aush`:\n\n```python\npoetry.init(\n    no_interaction = True,\n    name = args.name,\n    author = git.config(get=\'user.name\'),\n    dev_dependency = [\'pytest\', \'ipython\', \'pylint\', \'mypy\', \'black\', \'pudb\'],\n)\npoetry.install()\n```\n\nI think that finally feels *better* than either shell or Python alone. Feel free to look at [`create-python`\'s history](https://github.com/kbd/setup/commits/master/HOME/bin/create-python) where I convert it from shell to Python and try various libraries along the way.\n\n# Usage\n\n`aush` has `Command`s and `Result`s. You make a base command like:\n\n```\nfrom aush import echo, git, python3\n```\n\nThen, any new command is immutably created from another command by either dot or indexing notation:\n\n```python\ngit.init()\necho("*.sqlite3") > \'.gitignore\'\ngit.add(all=True)\ngit.commit(m="Initial commit")\n```\n\nHere\'s an example of explicitly creating a command from another using index notation:\n\n```python\ndjango_manage = python3["manage.py"]\ndjango_manage.startapp(args.name)\ndjango_manage.migrate()\ndjango_manage.createsuperuser(\n    noinput = True,\n    username = "admin",\n    email = git.config(get=\'user.email\'),\n    _env = dict(DJANGO_SUPERUSER_PASSWORD=\'password\'),\n)\n```\n\nHere\'s how that call to `createsuperuser` looks in equivalent shell:\n\n```shell\nDJANGO_SUPERUSER_PASSWORD=\'password\' python3 manage.py createsuperuser \\\n  --noinput \\\n  --username admin \\\n  --email "$(git config --get=user.email)"\n```\n\nNote that, like shell, in `aush` you don\'t have to manually merge in `os.environ` if you want to add new things to the environment.\n\nA program gets executed when a function call () is made on a `Command`.\nThat returns a `Result`.\nThe result has `.code`, `.stdout`, and `.stderr` members to get the results of running the command.\nUsing the `Result` you can also redirect stdout to a file with `result > \'filename\'`, `bool(result)` indicates success or failure of the command, and more.\n\n`aush` calls follow the following conventions:\n\n- positional arguments are passed verbatim\n- single-letter keyword arguments are converted to single-dash arguments\n- longer keyword arguments are converted to double-dash arguments\n- underscores in keywords and program names are converted to dashes\n- if the value of a keyword argument is an iterable (not `str`), it is repeated\n\nIf the conventions don\'t do what you want, you can always fall back to passing exactly the strings you want, positionally, in the list way of creating commands.\n\n# aush Design Goals:\n\n## `aush` should provide the *most convenient* way to write "shell scripts"\n\n* make it easy to call subprocesses and get their outputs\n* make subprocesses calls *look* like idiomatic Python\n* favor convenience over performance/control (there are lower level APIs available to use)\n* but still enable shell performance patterns, such as:\n  * piping and redirecting shouldn\'t have to wait for one process to complete in order to start\n  * aush can (optionally) log stdout/stderr "as they happen" same as you would for a shell script\n* Safer subprocess management\n  * be "safer" than shell scripts or Python\n  * fewer opportunities for string substitution or incorrect input handling leading to deadlocks\n  * throws exceptions on non-zero return codes (like set -e by default) to make it harder to miss error cases\n\n## Secondary design goals\n\n* it should make good use of asyncio, and eventually help script applications like expect\n* it should allow you to build tools around cli tools, like timestamping each line of output (something I miss from iTerm2)\n* just like you\'d use Pandas to deal with tabular data (I\'ll often use it just to massage csvs) I hope `aush` can be useful for calling subprocesses in your programs, and help write *safer* programs that interact with subprocesses, and bigger "shell scripts" than you would otherwise\n* Allow `aush` to be a command-line program itself that does something? Maybe when called as a module it can provide testing utilities like my `argv` or `randomtextgenerator`. Maybe an "aush script" could find a way to automatically import anything not found in python space and try to run it? That way you can avoid the \'from aush import a million things\' and turn logging on, etc.\n\n## Non-goals\n\n* `aush` is not a new language or an interactive shell; it\'s just a Python library\n\n# FAQ\n\nQ. Pronunciation?\n\nA. It\'s pronounced "awwsh" ("au" as in "audible").\n\nQ. Where does the name come from?\n\nA. Originally I named it "autoshell", but when I went to put it on PyPI I found "autoshell" was taken, while the shorter present name wasn\'t.\n',
    'author': 'Keith Devens',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kbd/aush',
    'py_modules': modules,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
